@@ -27,13 +27,13 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
   def self.instances
     # our regex for matching pkg_info output
     regex = /^(.*)-(\d[^-]*)[-]?(\w*)(.*)$/
-    fields = [:name, :ensure, :flavor ]
+    packages = []
 
     begin
       packages = pkginfo('-a')
       packages.split("\n").collect do |package|
         if match = regex.match(package.split[0])
-	  new( :name => match.captures[0],
+         new( :name => match.captures[0],
                :ensure => match.captures[1],
                :flavor => match.captures[2],
           )
@@ -64,21 +64,13 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
   end
 
   def latest
-    parse_pkgconf
-
-    if @resource[:source][-1,1] == ::File::SEPARATOR
-      e_vars = { 'PKG_PATH' => @resource[:source] }
-    else
-      e_vars = {}
-    end
-
     if @resource[:flavor]
       query = "#{@resource[:name]}--#{@resource[:flavor]}"
     else
       query = @resource[:name]
     end
 
-    output = Puppet::Util.withenv(e_vars) {pkginfo "-Q", query}
+    output = Puppet::Util.withenv({}) {pkginfo "-Q", query}
 
     if output.nil? or output.size == 0 or output =~ /Error from /
       debug "Failed to query for #{resource[:name]}"
@@ -117,52 +109,12 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
     self.install(true)
   end
 
-  def parse_pkgconf
-    unless @resource[:source]
-      if Puppet::FileSystem.exist?("/etc/pkg.conf")
-        File.open("/etc/pkg.conf", "rb").readlines.each do |line|
-          if matchdata = line.match(/^installpath\s*=\s*(.+)\s*$/i)
-            @resource[:source] = matchdata[1]
-          elsif matchdata = line.match(/^installpath\s*\+=\s*(.+)\s*$/i)
-            if @resource[:source].nil?
-              @resource[:source] = matchdata[1]
-            else
-              @resource[:source] += ":" + matchdata[1]
-            end
-          end
-        end
-
-        if @resource[:source][-1,1] != ::File::SEPARATOR
-          raise Puppet::Error,
-                "Please use full PKG_PATH notation instead of '#{@resource[:source]}'"
-        end
-        unless @resource[:source]
-          raise Puppet::Error,
-          "No valid installpath found in /etc/pkg.conf and no source was set"
-        end
-      else
-        raise Puppet::Error,
-        "You must specify a package source or configure an installpath in /etc/pkg.conf"
-      end
-    end
-  end
-
   def install(latest = false)
     cmd = []
 
-    parse_pkgconf
-
-    if @resource[:source][-1,1] == ::File::SEPARATOR
-      e_vars = { 'PKG_PATH' => @resource[:source] }
-      full_name = get_full_name(latest)
-    else
-      e_vars = {}
-      full_name = @resource[:source]
-    end
-
     cmd << '-r'
     cmd << install_options
-    cmd << full_name
+    cmd << get_full_name(latest)
 
     if latest
       cmd.unshift('-z')
@@ -170,7 +122,7 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
 
     # pkg_add(1) doesn't set the return value upon failure so we have to peek
     # at it's output to see if something went wrong.
-    output = Puppet::Util.withenv(e_vars) { pkgadd cmd.flatten.compact }
+    output = Puppet::Util.withenv({}) { pkgadd cmd.flatten.compact }
     require 'pp'
     pp output
     if output =~ /Can't find /
@@ -222,7 +174,7 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
 
       return master_version unless master_version == 0
       return '' if version == -1
-      raise Puppet::Error, "#{version} is not available for this package"
+      raise Puppet::Error, _("%{version} is not available for this package") % { version: version }
     end
   rescue Puppet::ExecutionFailure
     return nil
